package com.dzeio.openhealth.ui.steps

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.charts.Entry
import com.dzeio.charts.series.BarSerie
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.adapters.StepsAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentStepsHomeBinding
import com.dzeio.openhealth.utils.ChartUtils
import dagger.hilt.android.AndroidEntryPoint
import java.text.DateFormat
import java.util.Calendar
import java.util.Date
import java.util.Locale
import java.util.TimeZone

@AndroidEntryPoint
class StepsHomeFragment :
    BaseFragment<StepsHomeViewModel, FragmentStepsHomeBinding>(StepsHomeViewModel::class.java) {

    companion object {
        const val TAG = "${Application.TAG}/SHFragment"
    }

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentStepsHomeBinding =
        FragmentStepsHomeBinding::inflate

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewModel.init()

        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = StepsAdapter()
        adapter.onItemClick = {
//            findNavController().navigate(
//                WaterHomeFragmentDirections.actionNavWaterHomeToNavWaterEdit(
//                    it.id
//                )
//            )
        }
        recycler.adapter = adapter

        val chart = binding.chart

        // setup serie
        val serie = BarSerie(chart)

        chart.apply {
            ChartUtils.materielTheme(chart, requireView())
            yAxis.apply {
                onValueFormat = { value -> "${value.toInt()}" }
            }

            xAxis.apply {
                dataWidth = 604800000.0
                textPaint.textSize = 32f
                onValueFormat = onValueFormat@{
                    val formatter = DateFormat.getDateTimeInstance(
                        DateFormat.SHORT,
                        DateFormat.SHORT,
                        Locale.getDefault()
                    )
                    return@onValueFormat formatter.format(Date(it.toLong()))
                }

            }
        }

        viewModel.goal.observe(viewLifecycleOwner) {
            if (it != null) {
                chart.yAxis.addLine(it.toFloat())
                chart.refresh()
            }
        }

        viewModel.items.observe(viewLifecycleOwner) { list ->
            adapter.set(list)

            if (list.isEmpty()) {
                return@observe
            }

//            chart.scroller.zoomEnabled = false

            val entries: HashMap<Long, Entry> = HashMap()

            list.forEach {
                val cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"))
                cal.timeInMillis = it.timestamp

                cal.set(Calendar.HOUR, 0)
                cal.set(Calendar.AM_PM, Calendar.AM)
                val ts = cal.timeInMillis
                if (!entries.containsKey(ts)) {
                    entries[ts] = Entry((ts).toDouble(), 0F, chart.yAxis.goalLinePaint.color)
                }

                entries[ts]!!.y += it.value.toFloat()

                if (viewModel.goal.value != null) {
                    if (entries[ts]!!.y > viewModel.goal.value!!) {
                        entries[ts]!!.color = null
                    }
                } else {
                    entries[ts]!!.color = null
                }
            }

            serie.entries = ArrayList(entries.values)

            chart.xAxis.x = chart.xAxis.getXMin()


            chart.refresh()
        }
    }
}
