package com.dzeio.openhealth.utils

import android.view.View
import com.dzeio.charts.ChartView
import com.dzeio.charts.series.BarSerie
import com.dzeio.charts.series.LineSerie
import com.github.mikephil.charting.charts.BarLineChartBase
import com.github.mikephil.charting.charts.LineChart
import com.github.mikephil.charting.components.AxisBase
import com.github.mikephil.charting.components.Description
import com.github.mikephil.charting.components.XAxis
import com.github.mikephil.charting.data.BarLineScatterCandleBubbleData
import com.github.mikephil.charting.data.Entry
import com.github.mikephil.charting.data.LineDataSet
import com.github.mikephil.charting.formatter.ValueFormatter
import com.github.mikephil.charting.interfaces.datasets.IBarLineScatterCandleBubbleDataSet
import com.google.android.material.color.MaterialColors
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

/**
 * Utils object to create graphics using the mikephil charting library
 *
 * TODO: migrate to DzeioCharts once it is ready
 */
object ChartUtils {

    fun lineChartSetup(chart: LineChart, mainColor: Int, textColor: Int) {
        barLineChartSetup(chart, mainColor, textColor)
        // chart.isAutoScaleMinMaxEnabled = true
    }

    fun lineDataSet(lineDataSet: LineDataSet): LineDataSet {
        return lineDataSet.apply {
            setDrawCircles(false)
            setDrawCircleHole(false)
            mode = LineDataSet.Mode.HORIZONTAL_BEZIER
        }
    }

    private fun <T : BarLineScatterCandleBubbleData<out IBarLineScatterCandleBubbleDataSet<out Entry>>?> barLineChartSetup(
        chart: BarLineChartBase<T>,
        mainColor: Int,
        textColor: Int
    ) {
        chart.apply {
            // Setup
            legend.isEnabled = true
            description = Description().apply { isEnabled = false }

            xAxis.apply {
                valueFormatter = DateValueFormatter()
                position = XAxis.XAxisPosition.BOTTOM
                setDrawGridLines(false)
                setLabelCount(3, true)
                this.textColor = textColor
                // setDrawGridLines(false)
                // setDrawZeroLine(false)
                setDrawAxisLine(false)
                disableGridDashedLine()
                invalidateOutline()
            }

            axisLeft.apply {
                axisMinimum = 0f
                isEnabled = false
                axisLineColor = mainColor
                this.textColor = textColor
                setDrawZeroLine(false)
                setLabelCount(0, true)
                setDrawGridLines(false)
                setDrawBorders(false)
            }
            axisRight.apply {
                axisMinimum = 0f
                this.textColor = textColor
                setLabelCount(4, true)
            }
            setNoDataTextColor(textColor)

            legend.isEnabled = false
            isDragEnabled = true
            // isScaleYEnabled = false
            description = Description().apply { isEnabled = false }
            isScaleXEnabled = true
            setPinchZoom(false)
            setDrawGridBackground(false)
            setDrawBorders(false)
        }
    }

    class DateValueFormatter(
        private val transformer: Int = 1
    ) : ValueFormatter() {
        override fun getAxisLabel(value: Float, axis: AxisBase?): String {
            return SimpleDateFormat(
                "yyyy-MM-dd",
                Locale.getDefault()
            ).format(Date(value.toLong() * transformer))
            // return super.getAxisLabel(value, axis)
        }
    }

    /**
     * Apply Material theme to a DzeioChart [ChartView]
     */
    fun materielTheme(chart: ChartView, view: View) {

        val errorColor = MaterialColors.getColor(
            view,
            com.google.android.material.R.attr.colorError
        )

        chart.apply {
            yAxis.apply {
                textLabel.color = MaterialColors.getColor(
                    view,
                    com.google.android.material.R.attr.colorOnPrimaryContainer
                )
                linePaint.color = MaterialColors.getColor(
                    view,
                    com.google.android.material.R.attr.colorOnPrimaryContainer
                )
                goalLinePaint.color = errorColor
            }

            xAxis.apply {
                textPaint.color = MaterialColors.getColor(
                    view,
                    com.google.android.material.R.attr.colorOnPrimaryContainer
                )
            }

            for (serie in series) {
                if (serie is BarSerie) {
                    serie.apply {
                        barPaint.color = MaterialColors.getColor(
                            view,
                            com.google.android.material.R.attr.colorPrimary
                        )
                        textPaint.color = MaterialColors.getColor(
                            view,
                            com.google.android.material.R.attr.colorOnPrimary
                        )
                    }
                } else if (serie is LineSerie) {
                    serie.apply {
                        linePaint.color = MaterialColors.getColor(
                            view,
                            com.google.android.material.R.attr.colorPrimary
                        )
                        textPaint.color = MaterialColors.getColor(
                            view,
                            com.google.android.material.R.attr.colorOnPrimary
                        )
                    }
                }
            }
        }
    }
}
